from flask import Flask, render_template, request, flash
import secrets
from mango import Mango
from bson import ObjectId
from helpers import *

debug = True

app = Flask(__name__)
app.secret_key = secrets.token_hex(22)


@app.route('/location')
def locations():
    locs = mango.locations_get_locations()
    return render_template('locations.html', locations=locs)


@app.route('/location/<locid>')
def location(locid):
    loc = mango.locations_get_location(locid)
    return render_template('location.html', location=loc)


@app.route('/location/<locid>/config', methods=['GET', 'POST'])
def location_config(locid):
    if request.method == "POST":
        request_type = request.form.get('type')

        if request_type == 'enabled-stations':
            sta = request.form.getlist('station')
            mango.locations_update_enabled_stations(locid, sta)
        elif request_type == 'enable-requests':
            ena = bool(request.form.get('enable', default=False))
            mango.locations_update_enable_requests(locid, ena)
        elif request_type == 'enable-radio':
            ena = bool(request.form.get('enable', default=False))
            mango.locations_update_enable_radio(locid, ena)
        else:
            flash('You posted a form but no such call exists.')

    loc = mango.locations_get_location(locid)
    sta = mango.stations_get_stations()
    pla = mango.player_get_state(loc['config']['player'])

    off = check_player_offline(pla['last_update'])

    return render_template('location_config.html', location=loc, stations=sta, ObjectId=ObjectId, offline=off)


@app.route('/location/<locid>/request', methods=['GET', 'POST'])
def location_request(locid):
    loc = mango.locations_get_location(locid)
    return render_template('location_request.html', location=loc)


@app.route('/location/<locid>/radio', methods=['GET', 'POST'])
def location_radio(locid):
    loc = mango.locations_get_location(locid)
    sta = mango.stations_get_stations()
    pla = mango.player_get_state(loc['config']['player'])

    if request.method == "POST":
        request_type = request.form.get('type')

        if request_type == 'select-station':
            station = request.form.get('station')
            mango.player_set_radio(loc['config']['player'], station)
            flash('Enqueued change request')
        elif request_type == 'stop-stream':
            mango.player_pause_radio(loc['config']['player'])
            flash('Enqueued change request')
        else:
            flash('You posted a form but no such call exists.')

    # report current playing radio or false
    if pla['radio']['playing'] is False:
        cur = False
    else:
        cur = pla['radio']['stationId']

    # set location radio control config enabled
    enabled = loc['config']['enable_radio']

    # check if player has not been offline for too long
    if check_player_offline(pla['last_update']):
        enabled = False

    return render_template('location_radio.html', location=loc, stations=sta, current=cur, ObjectId=ObjectId, enabled=enabled)


@app.route('/players/<player>/config', methods=['POST'])
def players(player):
    data = request.get_json()
    new_state = mango.player_get_config(player)

    mango.player_update_state(player, data)

    if data['radio'].get('cookie') == new_state['radio']['cookie']:  # save time by comparing last change cookie
        # one caviat, the cookie may not match up when the last station was paused then unpaused
        return {}

    def generate_radio_return():  # define function to generate response
        station = str(new_state['radio']['station'])
        url = mango.stations_get_station(new_state['radio']['station'])['stream']['url']
        cookie = new_state['radio']['cookie']
        return {'type': 'radio', 'stationId': station, 'url': url, 'cookie': cookie}

    # checking for playing bools
    if new_state['radio']['playing'] is False and data['radio']['playing'] is True:
        return {'type': 'silence'}
    elif new_state['radio']['playing'] is True and data['radio']['playing'] is False:
        return generate_radio_return()

    # checking if station is correct
    if str(new_state['radio']['station']) != data['radio'].get('stationId'):
        return generate_radio_return()

    return {}  # last resort in case everything matched but cookie did not, and to not status 500


@app.route('/stations')
def stations():

    return {}


if __name__ == '__main__':
    mango = Mango('mongodb://root:test@192.168.66.113:27017')
    app.run(debug=debug, host='0.0.0.0')
