from pymongo import MongoClient
from bson.objectid import ObjectId
from secrets import token_hex

from helpers import *


class Mango:
    # holy shit, just found out a major bug, find_one and then replace_one is really bad with concurrency.
    def __init__(self, connect):
        try:
            self.client = MongoClient(connect)
            self.locations = self.client['radio']['locations']
            self.players = self.client['radio']['players']
            self.stations = self.client['radio']['stations']
        except ConnectionError:
            print('MongoDB connection error')

    def player_pause_radio(self, plaid):
        found = self.players.find_one({"_id": ObjectId(plaid)})
        if found is None:
            return False

        found['new_state']['radio']['playing'] = False
        found['new_state']['radio']['cookie'] = token_hex(8)

        self.players.replace_one({"_id": ObjectId(plaid)}, found)
        return True

    def player_set_radio(self, plaid, staid):
        found = self.players.find_one({"_id": ObjectId(plaid)})
        if found is None:
            return False

        if self.stations_get_station(staid):
            staid = ObjectId(staid)

        found['new_state']['radio']['playing'] = True
        found['new_state']['radio']['station'] = staid
        found['new_state']['radio']['cookie'] = token_hex(8)

        self.players.replace_one({"_id": ObjectId(plaid)}, found)
        return True

    def player_update_state(self, plaid, state):
        """
        :param plaid: str
        :param state: {'radio': {'playing': bool, 'url': str}, 'request': {'playing': bool, 'url': str}}
        :return: True on sucess, False on failure
        """
        found = self.players.find_one({"_id": ObjectId(plaid)})
        if found is None:
            return False

        state['last_update'] = utc_time_now()
        found['last_state'] = state

        self.players.replace_one({"_id": ObjectId(plaid)}, found)
        return True

    def player_get_state(self, plaid):
        found = self.players.find_one({"_id": ObjectId(plaid)})
        if found is None:
            return False

        return found['last_state']

    def player_get_config(self, plaid):
        found = self.players.find_one({"_id": ObjectId(plaid)})
        if found is None:
            return False

        return found['new_state']

    def locations_get_locations(self):
        locations = {}
        for x in self.locations.find({}):
            locations[str(x['_id'])] = x['name']
        return locations

    def locations_get_location(self, locid):
        locid = ObjectId(locid)
        found = self.locations.find_one({'_id': locid})
        if found is None:
            return False

        return found

    def locations_update_enable_radio(self, locid, ena):
        locid = ObjectId(locid)
        found = self.locations.find_one({'_id': locid})

        if found is None:
            return False

        found['config']['enable_radio'] = ena
        self.locations.replace_one({"_id": locid}, found)
        return True

    def locations_update_enable_requests(self, locid, ena):
        locid = ObjectId(locid)
        found = self.locations.find_one({'_id': locid})

        if found is None:
            return False

        found['config']['enable_requests'] = ena
        self.locations.replace_one({"_id": locid}, found)
        return True

    def locations_update_enabled_stations(self, locid, lst):
        sta = []
        locid = ObjectId(locid)
        found = self.locations.find_one({'_id': locid})

        if found is None:
            return False

        for x in lst:
            if self.stations_get_station(x):
                x = ObjectId(x)
                sta.append(x)

        found['config']['enabled_stations'] = sta
        self.locations.replace_one({"_id": locid}, found)
        return True

    def stations_get_stations(self):
        stations = self.stations.find({})
        stations_dict = {}

        for x in stations:
            i = str(x['_id'])
            x.pop('_id')
            stations_dict[i] = x

        return stations_dict

    def stations_get_station(self, staid):
        staid = ObjectId(staid)
        found = self.stations.find_one({'_id': staid})
        if found is None:
            return False

        return found


if __name__ == '__main__':
    mango = Mango('mongodb://root:test@192.168.66.113:27017')
    print(mango.stations_get_stations())
