import requests
import re

# variable
api = "https://CHANGEME:8080"  # change your url without the trailing slash, include port if needed and optionally add https
blacklist = '(Xunlei|-XL0012-).*'  # to append any clients just add more vertical symbols, this regex will match any client name that contains these strings


# constant, do not change
api_data = api + '/api/v2/sync/maindata'
api_peers = api + '/api/v2/sync/torrentPeers'
api_ban = api + '/api/v2/transfer/banPeers'
api_banlist = api + '/api/v2/app/preferences'


def get_active_torrents():
    """
        gets all torrents and returns a set with all the uploading torrent hashes
        this might miss some peers that have connected after this initial request
        but this will substantially decrease the runtime of the script
        because non uploading torrents are not queried for any peers
    """
    torrents = set()
    
    r = requests.get(api_data)  # request data
    
    if r.status_code != 200:  # if requests not successful
        print(f'Ehh api error or something? Could not get the list of torrents got HTTP status code {r.status_code} instead.')
        exit(1)  # exit script do not continue
    
    maindata = r.json()  # store all data in maindata
    
    for torrent in maindata['torrents']:  # for every torrent hash
        if maindata['torrents'][torrent]['state'] == 'uploading':  # if torrent state is uploading
            torrents.add(torrent)  # add torrent hash to torrents set
    
    return torrents  # return torrents set

def get_peers_from_torrent(hash):
    """
        hash: string value of a torrent hash
        returns a dict with all the peers connected to the torrent
    """
    r = requests.get(api_peers, params={'hash': hash})  # request all peers from torrent hash
    
    if r.status_code != 200:  # if requests not successful
        print(f'got http status code {r.status_code} while querying peers of torrent {hash} exiting script')
        exit(1)  # exit script do not continue
    
    return r.json()  # return json of all peers from torrent hash

def ban_peer(peer):
    """
        peer: string value containing IP:PORT
        returns True when peer is added to banlist otherwise False
    """
    r = requests.post(api_ban, data={'peers': peer})  # send post request with peerset
    
    if r.status_code != 200:  # post request failed
        return False
    else:
        return True

def main():
    """
        main processing function
    """

    for torrent in get_active_torrents():  # for every active uploading torrent
            peers = get_peers_from_torrent(torrent)['peers']  # get peers from torrent
            for peer in peers:  # for every peer('ip:port')
                client = peers[peer]['client']  # save client name in client
                if bool(re.search(blacklist, client)):  # if client name is in blacklist
                    # wanted to print client name but randomized -XL0012- names include unicode, i found out the hard way that you shouldn't
                    print(f"Found Xunlei client with IP {peer} trying to add to banlist")
                    if ban_peer(peer):  # call ban_peer() with ipset
                        print('    added peer to banlist')
                    else:  # ban_peer() failed, tough you should never see this message since the script sould have exited already at this point
                        print('    Failed to add peer to banlist, please do a checkup yourself')

if __name__ == "__main__":
    main()