from flask import Flask, render_template, redirect, request, url_for
import secrets, requests, os
from ssh import ssh_handler
from db import conn

app = Flask(__name__)
debug = True


def sign_user(key, username, description):
    ssh = ssh_handler.SshHandler()
    ca = db.select_ca()
    ssh.load_ca(ca[2], ca[3])
    ssh.load_key(key)
    try:
        signature = ssh.sign_user(username, description)
        db.insert_user(username, key, description, signature)
        return signature
    except:
        return False
        
def sign_host(key, principals, description):
    ssh = ssh_handler.SshHandler()
    ca = db.select_ca()
    ssh.load_ca(ca[2], ca[3])
    ssh.load_key(key)
    try:
        signature = ssh.sign_host(principals, description)
        db.insert_host(principals, key, description, signature)
        return signature
    except:
        return False

@app.route('/')
def index():
    hosts = db.select_hosts()
    users = db.select_users()
    return render_template('index.html', hosts=hosts, users=users)

@app.route('/api/sign/host', methods=['POST'])
def api_sign_host():
    key = request.form.get('key')
    principals = request.form.get('principals')
    description = request.form.get('description')
    submit = request.form.get('submit')

    if key is None or principals is None or description is None:
        return 'Missing required parameters', 400
    
    if len(key) == 0 or len(principals) == 0 or len(description) == 0:
        return 'Missing required parameters', 400
    
    r = sign_host(key, principals, description)
    
    if not r:
        print(f'{key} -- {principals} -- {description}', flush=True)
        return 'Error signing key', 500
    
    if submit == 'Sign':
        return redirect(url_for('index'))
    
    return r
    
@app.route('/api/sign/user', methods=['POST'])
def api_sign_user():
    key = request.form.get('key')
    username = request.form.get('username')
    description = request.form.get('description')
    submit = request.form.get('submit')
    
    if key is None or username is None or description is None:
        return 'Missing required parameters', 400
        
    if len(key) == 0 or len(username) == 0 or len(description) == 0:
        return 'Missing required parameters', 400
    
    r = sign_user(key, username, description)
    
    if not r:
        return 'Error signing key', 500
    
    if submit == 'Sign':
        return redirect(url_for('index'))
    
    return r
    

@app.route('/api/generate/ca', methods=['POST'])
def api_generate_ca():
    ssh = ssh_handler.SshHandler()
    gen = ssh.generate_ca()
    db.insert_ca(gen['public'], gen['private'])
    return 'Ok'


@app.route('/api/print/ca', methods=['GET'])
def api_print_ca():
    return str(db.select_ca()[2].decode('utf-8'))


if __name__ == '__main__':
    db = conn.Mydb()
    app.run(debug=debug, host='0.0.0.0', port=5001)