import subprocess
import secrets
import os
import shutil
import cmd
import sys

root = 'D:/sshCA-flask/temp'

class SshHandler():
    def __init__(self):
        self.workfolder = f'{root}/s-{secrets.token_hex(8)}'
        #self.workfolder = f'{root}/s-bruh'
        os.mkdir(self.workfolder)
        
        self.ca = False
        self.cert = False

    def generate_ca(self, alg='ed25519', comment=secrets.token_hex(8), passphrase=''):
        if self.ca:
            return False
            
        options = ['ssh-keygen', 
                   '-q', 
                   '-f', f'{self.workfolder}/CA', 
                   '-t', alg, 
                   '-C', comment, 
                   '-N', passphrase]
        ret = subprocess.check_call(options)
        
        ca = {}
        
        with open(f'{self.workfolder}/CA', 'rb') as file:
            ca['private'] = file.read()
            
        with open(f'{self.workfolder}/CA.pub', 'rb') as file:
            ca['public'] = file.read()
        
        self.ca = True
        return ca
    
    def load_ca(self, public, private):
        if self.ca:
            return False
        
        with open(f'{self.workfolder}/CA', 'wb') as file:
            file.write(private)
            
        with open(f'{self.workfolder}/CA.pub', 'wb') as file:
            file.write(public)
        
        self.ca = True
        return True
    
    def load_key(self, cert):
        if self.cert:
            return False
            
        with open(f'{self.workfolder}/key.pub', 'w') as file:
            file.writelines(cert)
        
        self.cert = True
        return True
        
    def sign_user(self, username, description, validity=3650):
        if not self.cert or not self.ca:
            return False

        options = ['ssh-keygen', 
                   '-q', 
                   '-s', f'{self.workfolder}/CA', 
                   '-I', description, 
                   '-n', username, 
                   #'-V', str(validity),
                   f'{self.workfolder}/key.pub']
        ret = subprocess.check_call(options)

        with open(f'{self.workfolder}/key-cert.pub', 'r') as file:
            return file.readline()
            
    def sign_host(self, hosts, description, validity=3650):
        if not self.cert or not self.ca:
            return False

        options = ['ssh-keygen', 
                   '-q', 
                   '-h', 
                   '-s', f'{self.workfolder}/CA', 
                   '-I', description, 
                   '-n', hosts, 
                   #'-V', str(validity),
                   f'{self.workfolder}/key.pub']
        ret = subprocess.check_call(options)

        with open(f'{self.workfolder}/key-cert.pub', 'r') as file:
            return file.readline()
    
    
    def __del__(self):
        shutil.rmtree(self.workfolder)
        pass
        
        
if __name__ == '__main__':
    ssh = SshHandler()
    print(ssh.generate_ca())