import requests  # just do: pip install requests | if you dont have it
import json
import secrets
from datetime import date

# change variables below
TRUENAS_KEY = '1-xxxxxx'
TRUENAS_ADDRESS = 'host.domain.tld'
TRUENAS_PROTOCOL = 'https'
TRUENAS_PORT = '443'
TRUENAS_API_PATH = '/api/v2.0'
LOCAL_CERTIFICATE = 'fullchain.cer'
LOCAL_KEY = 'truenas.key'
CERT_NAME_PREFIX = 'automatic'

# Do not change below
requests.packages.urllib3.disable_warnings(category=InsecureRequestWarning)
CONNECT_DOMAIN = f'{TRUENAS_PROTOCOL}://{TRUENAS_ADDRESS}:{TRUENAS_PORT}{TRUENAS_API_PATH}'
NEW_CERT_NAME = f'{CERT_NAME_PREFIX}_{date.today()}_{secrets.token_hex(4)}'
HEADERS = requests.structures.CaseInsensitiveDict()
HEADERS["Accept"] = "application/json"
HEADERS["Content-Type"] = "application/json"
HEADERS["Authorization"] = f"Bearer {TRUENAS_KEY}"

def load_certificates_and_keys():
    """
    returns tuple with cert on 0(str) and key on 1(str)
    """
    with open(LOCAL_KEY, 'r') as file:
        priv_key = file.read()
    with open(LOCAL_CERTIFICATE, 'r') as file:
        full_chain = file.read()
    return (full_chain, priv_key)

def list_installed_certificates():
    """
    Returns a dict with the cert id's(int) as keys and values(str) the names of the certs in the truenas system
    """
    certs = {}
    
    r = requests.get(f'{CONNECT_DOMAIN}/certificate', headers=HEADERS)
    
    if r.status_code != 200:
        print(f'Requests exited with status code {r.status_code}')
        return 
       
    for x in r.json():
        certs[x['id']] = x['name']
        
    return certs

def put_certificate(c, k):
    """
    Puts given certificate and key in truenas system, returns certificate id(int) as in truenas
    """
    data = json.dumps({
        "create_type": "CERTIFICATE_CREATE_IMPORTED",
        "name": NEW_CERT_NAME,
        "certificate": c,
        "privatekey": k
    })
    
    r = requests.post(f'{CONNECT_DOMAIN}/certificate', headers=HEADERS, data=data)
    
    if r.status_code != 200:
        print(f'Requests exited with status code {r.status_code}')
        return 
    
    installed_certificates = list_installed_certificates()
    
    for x in installed_certificates:
        if installed_certificates[x] == NEW_CERT_NAME:
            return x

def set_active_certificate(i):
    data = json.dumps({
        "ui_certificate": i
    })
    
    r = requests.put(f'{CONNECT_DOMAIN}/system/general', headers=HEADERS, data=data)
    
    if r.status_code != 200:
        print(f'Requests exited with status code {r.status_code}')
        return
    else:
        return True

def restart_ui():
    r = requests.post(f'{CONNECT_DOMAIN}/system/general/ui_restart', headers=HEADERS)
    
    if r.status_code != 200:
        print(f'Requests exited with status code {r.status_code}')
        return
    else:
        return True

if __name__ == "__main__":
    cert, key = load_certificates_and_keys()
    id = put_certificate(cert, key)
    set_active_certificate(id)
    restart_ui()


# todo:
# remove old certificates, currently new certificates are added but not removed